# Brainstorm application for concept-mapping
# All code copyright activityworkshop.net, see license.txt file

# Class for saving storms as .storm file
class StormSaver:

	# Save the given storm to the given filename
	def saveStorm(self, storm, filename, savePos):
		outfile = open(str(filename), 'w')
		# Should work out encoding really - don't know how
		outfile.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
		outfile.write("<brainstorm version=\"1\">\n")    # version number hardcoded here :(
		# header
		outfile.write("\t<header>\n\t\t<title>%s</title>\n\t\t<author>%s</author>\n\t</header>\n" % (storm.title, storm.author))
		# nodes
		outfile.write("\t<nodes>\n")
		for node in storm.nodes:
			position = "posx=\"%d\" posy=\"%d\"" % (node.pos().x(), node.pos().y()) if savePos else ""
			outfile.write("\t\t<node id=\"%s\" label=\"%s\" %s/>\n" % (node.nodeid, node.label, position))
			# TODO: need to protect certain characters such as <, >, \?
		outfile.write("\t</nodes>\n")
		# connections
		outfile.write("\t<connections>\n")
		for edge in storm.edges:
			outfile.write("\t\t<connection source=\"%s\" dest=\"%s\" />\n" %(edge.source.nodeid, edge.dest.nodeid))
		outfile.write("\t</connections>\n")
		outfile.write("</brainstorm>\n")
		outfile.close()



# Class for saving storms as svg file
class SvgExporter:

	# Export the given storm to the given filename
	def exportSvg(self, storm, filename):
		outfile = open(str(filename), 'w')
		# Should work out encoding really - don't know how
		outfile.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
		outfile.write("<svg xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns=\"http://www.w3.org/2000/svg\" version=\"1.0\" width=\"600\" height=\"400\">\n")
		outfile.write("\t<g id=\"layer1\">\n")

		# TODO: maybe need to calculate scale factor as well as mean position
		xpos = [float(node.pos().x()) for node in storm.nodes]
		ypos = [float(node.pos().y()) for node in storm.nodes]
		xoffset = 300 - sum(xpos)/len(xpos)
		yoffset = 200 - sum(ypos)/len(ypos)

		# connections
		for conn in storm.edges:
			x1 = conn.source.pos().x() + xoffset
			y1 = conn.source.pos().y() + yoffset
			x2 = conn.dest.pos().x() + xoffset
			y2 = conn.dest.pos().y() + yoffset
			outfile.write("\t\t<line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" stroke=\"#000000\" stroke-width=\"3\" />\n" % (x1, y1, x2, y2))

		# node bubbles and texts
		for node in storm.nodes:
			nodex = node.pos().x() + xoffset
			nodey = node.pos().y() + yoffset
			outfile.write("\t\t<ellipse transform=\"translate(%d %d)\"  rx=\"%d\" ry=\"%d\" fill=\"#fefefe\" stroke=\"#000000\" stroke-width=\"1\" />" % (nodex, nodey, int(node.boundingRect().width()/2), int(node.boundingRect().height()/2)))
			fontweight = "bold" if node.font().bold() else "normal"
			# TODO: need to protect certain characters such as <, >, \?
			outfile.write("\t\t<text x=\"%d\" y=\"%d\" text-anchor=\"middle\" font-size=\"%d\" font-weight=\"%s\">%s</text>\n" % (nodex, nodey, node.font().pointSize(), fontweight, node.label))

		outfile.write("\t</g>\n</svg>\n")
		outfile.close()

